@@ -1,35 +1,34 @@
 (**************************************************************************)
 (*                                                                        *)
-(*    Copyright 2012-2013 OCamlPro                                        *)
+(*    Copyright 2012-2015 OCamlPro                                        *)
 (*    Copyright 2012 INRIA                                                *)
 (*                                                                        *)
-(*  All rights reserved.This file is distributed under the terms of the   *)
-(*  GNU Lesser General Public License version 3.0 with linking            *)
-(*  exception.                                                            *)
-(*                                                                        *)
-(*  OPAM is distributed in the hope that it will be useful, but WITHOUT   *)
-(*  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY    *)
-(*  or FITNESS FOR A PARTICULAR PURPOSE.See the GNU General Public        *)
-(*  License for more details.                                             *)
+(*  All rights reserved. This file is distributed under the terms of the  *)
+(*  GNU Lesser General Public License version 2.1, with the special       *)
+(*  exception on linking described in the file LICENSE.                   *)
 (*                                                                        *)
 (**************************************************************************)
 
-(** Cudf interface *)
+(** Solver interaction through Cudf, conversion of solver answer to an opam
+    solution *)
 
 open OpamTypes
 
 (** Cudf sets *)
-module Set: OpamMisc.SET with type elt = Cudf.package
+module Set: OpamStd.SET with type elt = Cudf.package
 
 (** Cudf maps *)
-module Map: OpamMisc.MAP with type key = Cudf.package
+module Map: OpamStd.MAP with type key = Cudf.package
 
 (** Cudf graph *)
 module Graph: sig
   (** Graph of cudf packages *)
-  type t
 
-  (** Build a graph from a CUDF universe *)
+  include module type of Algo.Defaultgraphs.PackageGraph.G
+
+  (** Build a graph from a CUDF universe. Warning: dependency edges are towards
+      the dependency, which is the reverse of what happens in the action
+      graph. *)
   val of_universe: Cudf.universe -> t
 
   (** Return the transitive closure of [g] *)
@@ -38,29 +37,17 @@ module Graph: sig
   (** Return the transitive closure of dependencies of [set],
       sorted in topological order. *)
   val close_and_linearize: t -> Set.t -> Cudf.package list
+
+  (** Reverse the direction of all edges *)
+  val mirror: t -> t
 end
 
-(** Difference between universes *)
-module Diff: sig
-
-  (** Differences between the versions of a given package *)
-  type package = {
-    installed  : Set.t;
-    removed    : Set.t;
-    reinstalled: Set.t;
-  }
-
-  (** Difference between universe *)
-  type universe = (Cudf_types.pkgname, package) Hashtbl.t
-
-  (** Computation of differences between universe *)
-  val diff: Cudf.universe -> Cudf.universe -> universe
-
-end
+(** Computation of differences between universe. Returns the sets of packages to
+    install and remove respectively. *)
+val diff: Cudf.universe -> Cudf.universe -> (Set.t * Set.t)
 
 (** Cudf action graph *)
-module ActionGraph: OpamParallel.GRAPH with type V.t = Cudf.package action
-type solution = (Cudf.package, ActionGraph.t) gen_solution
+module ActionGraph: OpamActionGraph.SIG with type package = Cudf.package
 
 (** Abstract type that may be returned in case of conflicts *)
 type conflict
@@ -91,26 +78,39 @@ val get_final_universe:
 
 (** Compute the list of actions to match the difference between two
     universe. Remark: the result order is unspecified, ie. need to use
-    [solution_of_actions] to get a solution which respects the
+    [atomic_actions] to get a solution which respects the
     topological order induced by dependencies. *)
-val actions_of_diff: Diff.universe -> Cudf.package action list
+val actions_of_diff:
+  (Set.t * Set.t) -> Cudf.package atomic_action list
 
 exception Cyclic_actions of Cudf.package action list list
 
 (** Computes the actions to process from a solution, from the actions
     obtained by a simple universe diff. The 'simple' universe
     should not contain build dependencies and will be used for resolution ;
-    [complete_universe] should include build-deps, it's used for ordering
-    of actions and, together with the [requested] set of package names,
-    for computing the reasons of the actions.
+    [complete_universe] should include build-deps, it's used to get the
+    dependency ordering of actions.
+
+    Returns a graph of atomic actions, i.e. only removals and installs. Use
+    [reduce_actions] to reduce it to a graph including reinstall and
+    up/down-grade actions.
 
     May raise [Cyclic_actions]. *)
-val solution_of_actions:
+val atomic_actions:
   simple_universe:Cudf.universe ->
   complete_universe:Cudf.universe ->
-  requested:OpamPackage.Name.Set.t ->
-  Cudf.package action list ->
-  solution
+  [< Cudf.package highlevel_action ] list ->
+  ActionGraph.t
+
+(** Heuristic to compute the likely cause of all actions in a graph from the set
+    of packages passed in the original request. Assumes a reduced graph. Takes
+    the set of requested package names, and the set of packages marked for
+    reinstall. *)
+val compute_root_causes:
+  ActionGraph.t -> OpamPackage.Name.Set.t -> OpamPackage.Set.t ->
+  Cudf.package cause Map.t
+
+exception Solver_failure of string
 
 (** Resolve a CUDF request. The result is either a conflict holding
     an explanation of the error, or a resulting universe.
@@ -132,7 +132,7 @@ val to_actions:
   (Cudf.universe -> Cudf.universe) ->
   Cudf.universe ->
   (Cudf.universe, conflict) result ->
-  (Cudf.package action list, conflict) result
+  (Cudf.package atomic_action list, conflict) result
 
 (** [remove universe name constr] Remove all the packages called
     [name] satisfying the constraints [constr] in the universe
@@ -153,32 +153,56 @@ val remove_all_uninstalled_versions_but: Cudf.universe ->
 
 (** Cudf labels for package fields in the cudf format
     (use for the field Cudf.pkg_extra and with Cudf.lookup_package_property) *)
-val s_source: string         (** the original OPAM package name (as string) *)
-val s_source_number: string  (** the original OPAM package version (as string) *)
-val s_reinstall: string      (** a package to be reinstalled (a bool) *)
-val s_installed_root: string (** true if this package belongs to the roots
-                                 ("installed manually") packages *)
-val s_pinned: string         (** true if the package is pinned to this version *)
+
+(** the original OPAM package name (as string) *)
+val s_source: string
+
+(** the original OPAM package version (as string) *)
+val s_source_number: string
+
+(** a package to be reinstalled (a bool) *)
+val s_reinstall: string
+
+(** true if this package belongs to the roots ("installed manually")
+    packages *)
+val s_installed_root: string
+
+(** true if the package is pinned to this version *)
+val s_pinned: string
+
+(** the number of versions of the package since this one, cubed *)
+val s_version_lag: string
 
 (** {2 Pretty-printing} *)
 
 (** Convert a package constraint to something readable. *)
 val string_of_vpkgs: Cudf_types.vpkg list -> string
 
-val make_conflicts: Cudf.universe -> Algo.Diagnostic.diagnosis -> ('a, conflict) result
-val cycle_conflict: Cudf.universe -> string list list -> ('a, conflict) result
+val make_conflicts:
+  version_map:int package_map -> Cudf.universe ->
+  Algo.Diagnostic.diagnosis -> ('a, conflict) result
+val cycle_conflict:
+  version_map:int package_map -> Cudf.universe ->
+  string list list -> ('a, conflict) result
 
-(** Convert a conflict to something readable by the user. The first argument
-    should return a string like "lwt<3.2.1 is not available because..." when called
-    on an unavailable package (the reason can't be known this deep in the solver) *)
-val string_of_conflict: (atom -> string) -> conflict -> string
+(** Convert a conflict to something readable by the user. The second argument
+    should return a string explaining the unavailability, or the empty string,
+    when called on an unavailable package (the reason can't be known this deep
+    in the solver) *)
+val string_of_conflict:
+  package_set -> (name * OpamFormula.version_formula -> string) -> conflict ->
+  string
 
 (** Returns three lists of strings:
     - the final reasons why the request can't be satisfied
     - the dependency chains explaining it
     - the cycles in the actions to process (exclusive with the other two) *)
 val strings_of_conflict:
-  (atom -> string) -> conflict -> string list * string list * string list
+  package_set -> (name * OpamFormula.version_formula -> string) -> conflict ->
+  string list * string list * string list
+
+val conflict_chains:
+  package_set -> conflict -> (name * OpamFormula.version_formula) list list
 
 (** Dumps the given cudf universe to the given channel *)
 val dump_universe: out_channel -> Cudf.universe -> unit
@@ -201,13 +225,6 @@ val cudf2opam: Cudf.package -> package
 (** Returns the list of packages in a Cudf universe *)
 val packages: Cudf.universe -> Cudf.package list
 
-(** {2 External solver} *)
-val external_solver_available: unit -> bool
-
-(** Runs a test to check the version of the optimisation criteria accepted by
-    the external solver. Result is cached for subsequent queries. *)
-val check_cudf_version: unit -> [`Compat | `Latest]
-
 (** Converts an OPAM request to a Cudf request *)
 val to_cudf: Cudf.universe -> Cudf_types.vpkg request
   -> Cudf.preamble * Cudf.universe * Cudf.request
